#!/bin/sh

# this is for atmega168 (round)
# TODO: many things still left from v3.x (atmega128) that are inappropriate)

# notes about lock and fuses 

# lock=3f (or FF) means allow write anything (including bootloader)
# that's what we want

# --------------
function config_to_ecm
{
	echo "uploading config and tables. You might want to reboot (prog.pl vems.hex Etf)"
	make mtt
	cat etc/config.mtt >$SERIAL_DEV
	# the USB-RS232 happily swallows the first byte (if conn was idle for long)
	echo "Man" >$SERIAL_DEV
	cat etc/tables.mtt >$SERIAL_DEV
}

function config_from_ecm
{
	echo "verifying config and tables"
	prog.pl vems.hex Etd
	diff -i -B -b etc/config.txt mcd.txt 
	diff -i -B -b etc/tables.txt mct.txt 
	echo "========LCD content:========"
	cat mll.txt
}

# ---------------

if [ "$2" = "" ]
then
	serial=1
	echo "WARNING!!! serial number not specified, using $serial"
else
	serial=$2
fi

if [ -z "$COM_PORT" ]
then
	SERIAL_DEV=/dev/ttyS0
else
	SERIAL_DEV=$COM_PORT
fi

incdir=inc
mainnoid=main.hex
main=main-id.hex
# main=main.hex

perl patch_identity.pl $serial < $mainnoid >$main

[ -d $incdir ] || mkdir $incdir

efuse=$incdir/efuse.hex
hfuse=$incdir/hfuse.hex
lfuse=$incdir/lfuse.hex
lock=$incdir/lock.hex

Ufv="-U flash:v:$main"
Uev="-U efuse:v:$efuse"
Uhv="-U hfuse:v:$hfuse"
Ulv="-U lfuse:v:$lfuse"
Ulockv="-U lock:v:$lock"

Ufusesv="$Uev $Uhv $Ulv $Ulockv"

Ufw="-U flash:w:$main"
Uew="-u -U efuse:w:$efuse"
Uhw="-u -U hfuse:w:$hfuse"
Ulw="-u -U lfuse:w:$lfuse"
Ulockw="-u -U lock:w:$lock"

# DO NOT USE in SAME COMMAND, call avrdude SEPARATELY for EACH
# lock is not necessary (should be good by default; don't touch anyway, until after everything OK!)
# Ufusesw="$Uew $Uhw $Ulw $Ulockw"
Ufusesw="$Uew $Uhw $Ulw"

# makes xfuse.hex file according to parameters if it does not exist
function make_fusehex()
{
	if [ ! -f "$1" ]
	then
		echo ":01000000"$2 > $1
		echo ":00000001FF" >> $1
	fi
}

# simple checksum: sum of bytes (stored as an extra byte) is FF
make_fusehex $lfuse D728
make_fusehex $hfuse DC23
make_fusehex $efuse F807
make_fusehex $lock 3FC0

avrdudeargs="-c stk200 -p m168 -v"
# read from a known good (setup) AVR:
#avrdude -c stk200 -p m168 -v -U efuse:r:$efuse -U hfuse:r:$hfuse -U lfuse:r:$lfuse 

# lock bits = 0xef prevents selfdestruction of bootloader

if [ "$1" = "w" ]
then
	echo "erase and writing boot + write fuses after:"
	avrdude $avrdudeargs -e $Ufw 
	# $Ufusesw
fi

if [ "$1" = "f" ]
then
	echo "write fuses only:"
	# avrdude is stupid, so don't write more than 1 fuses in one step
  avrdude $avrdudeargs $Uew
  avrdude $avrdudeargs $Uhw
  avrdude $avrdudeargs $Ulw
# rethink thrice how and when to write lockbits!!!
#  avrdude $avrdudeargs $Ulockw 
fi

if [ "$1" = "v" ]
then
	echo "verify fuses and boot:"
	avrdude $avrdudeargs $Ufusesv $Ufv
fi

if [ "$1" = "vf" ]
then
	echo "verify fuses:"
	avrdude $avrdudeargs $Ufusesv 
fi

if [ "$1" = "e" ]
then
	echo "chip erase, unlock:"
  avrdude $avrdudeargs -e
fi

if [ "$1" = "b" ]
then
	echo "verifying bootloader"
	# prog.pl is not very tolerant at the start-to-talk phase
	# without the 't' flag, comm just did not work with my new pl2303 USB-rs232 adapter.
	# With 't' (that really just sends 9600 baud garbage 
	# when the bootloader is already in 19200) it works.
	# very strange,  
	perl bin/prog.pl $main tf
fi

if [ "$1" = "c" ]
then
	echo "uploading config"
	make mtt
	cat etc/config.mtt >$SERIAL_DEV
fi

if [ "$1" = "t" ]
then
	echo "uploading tables"
	make mtt
	cat etc/tables.mtt >$SERIAL_DEV
fi

if [ "$1" = "ct" ]
then
	config_to_ecm
fi

if [ "$1" = "cv" ]
then
	config_from_ecm
fi

if [ "$1" = "ctv" ]
then
	config_to_ecm
	config_from_ecm
fi

# THE END

