#!/usr/bin/perl -w

# MegasquirtAvr 2003
# wbo2 packet log data upload

$buf_len = 58;
$buf_len_short = 10; # why not 9 ? because of \r

$bootl_speed = 9600;

$display_file = undef;

if (@ARGV < 1) {
    usage();
}

sub usage
{
    	print "usage:\n";
    	print "\tperl bin/wbo2log.pl [:port]\n";
	print "[:port]\n";
	print "\t:COM1, :/dev/ttyS0\n\n";

    undef $ob if defined $ob;
    exit;
}

use vars qw( $OS_win $ob $port );

#$maxerr = 59;
$bootl_speed = 9600;
$timeout = 200;

BEGIN {
$OS_win = ($^O eq "MSWin32") ? 1 : 0;
if ($OS_win) {
  eval "use Win32::SerialPort";
  die "$@\n" if ($@);
} else {
  eval "use Device::SerialPort";
  die "$@\n" if ($@);
}
} # BEGIN


# Install like this: perl -MCPAN -e 'install Device::SerialPort'
# If you are running Windows 95 or later, you want the Win32::SerialPort
# module instead of this one. It has a compatible user interface.

process_param();

#print STDERR "using serialport:$port\n";
if($OS_win){
	$port = 'COM1' if !defined $port;
	$ob = Win32::SerialPort->new ($port);
} else {
	$port = '/dev/ttyS0' if !defined $port;
	$ob = Device::SerialPort->new ($port);
}
################################################################


serial_init();

if(defined $display_file )
{
	fileRead($display_file);
}
###############################################################

sub serial_init
{
# found on http://members.aol.com/Bbirthisel/alpha.html
# should work on multiple platforms

die "Can't open serial port $port: $^E\n" unless ($ob);

# save the state of the comm port for later
$old_baud = $ob->baudrate;
$old_parity = $ob->parity;
$old_data = $ob->databits;
$old_stop = $ob->stopbits;
$old_hshake = $ob->handshake;

# try using saved configuration

$ob->baudrate($bootl_speed);
$ob->parity('none');
$ob->databits(8);
$ob->stopbits(1);
$ob->handshake('none');
$ob->binary(T);

# without this communication returned too few bytes sometimes
$ob->read_interval(100) if ($OS_win);

$ob->write_settings || undef $ob;

$tmout = $timeout;
# ok, now port is configured
}

##############################################################################

$ob->baudrate($bootl_speed); # msavr speed
$ob->lookclear();
$ob->read_const_time(500);
comm_send('Man'); 
comm_send('mxc'); 
$b = srl_wait(8);
if ($b =~ /(Hello)|(enuPin)/) {
#    print "AVR said hello\n";
} else {
    print "No hello from AVR: $b \n";
}


	if( is_adc_test() ){
		do_adc_test();
	} elsif( is_wbo2_PIDlog() ){
		wbo2_PID_cycle();
		exit ;
	} else {
		wbo2_samples( 'mde03' );
		leave_comm();
		plot();
	}

# ==================
sub wbo2_samples
{
	my $cmd = shift;
	comm_send($cmd);
	%rimm = ();
	%dcmm = ();
	%himm = ();
	%pimm = ();
	%hcmm = ();
	min_max_reset(\%rimm);
	min_max_reset(\%dcmm);
	min_max_reset(\%himm);
	min_max_reset(\%pimm);
	min_max_reset(\%hcmm);

	$b = srl_wait(13 * $numbuf + 12);	# TODO: fix the 4096 byte limit...

	# if( $b =~ /\nW\=/)
	# capital W also allowed
	my @a = split /\n[Ww]\=/, $b;	# Beware: the \r stays on at the end !
	pop @a;
	foreach $samplebuf (@a){
		my $lenb = length($samplebuf);
		next if $lenb == 0;
		if($lenb == $buf_len){
			processbuffer($samplebuf);
		} elsif( $lenb == $buf_len_short ){
			@array= split(//, $samplebuf);
			processbuffer_short( 0 );
		} else {
			print STDERR "invalid buffer, length = $lenb\n";
		}
	}
} # sub wbo2_samples
# ======
sub leave_comm
{
comm_send($mde_exit);
printf("\n");
sleep(1);
$ob->purge_rx();
comm_send('bye');
$b = srl_wait(5);
if ($b =~ /Bye/) {
#   print "AVR said bye\n";
} else {
    print "No bye from AVR\n";
}

###############################################x

# restore state of comm port
$ob->baudrate($old_baud);
$ob->parity($old_parity);
$ob->databits($old_data);
$ob->stopbits($old_stop);
$ob->handshake($old_hshake);

undef $ob if defined $ob;

} # sub leave_comm


# ============
sub do_adc_test
{
	my $i;
	my $cnt = 3;
	for( $i=$cnt; $i>0; $i--){	
		comm_send($mde_exit);
		$samplebuf = srl_wait(100);	# TODO: fix the 4096 byte limit...
		if( $i == $cnt ){
			# skip leading garbage
			next;
		}
		my $lenb = length($samplebuf);
		if( $lenb == 0 ){
			print STDERR "Warning: received bytecount is 0 bytes, LEAVING\n";
			last;
		} elsif( $lenb != 32 ){
			print STDERR "Warning: received bytecount is NOT 32 bytes\n";
		}
		@array= split(//, $samplebuf);
		processbuffer_adc( 0x0D ); # TODO: make it dynamic, eg. mdx187 means comm(mdx18) and offset 7
	}
	exit;	
}

# ============
sub is_adc_test
{
	# print "mde_exit: $mde_exit\n";
	if( $mde_exit =~ /mdx/ ){
		return 1;
	}
	
	return undef;
}
# --
sub is_wbo2_PIDlog
{
	if( $mde_exit =~ /mdP/ ){
		return 1;
	}
	
	return undef;
}
# ==================
# plot data
sub plot
{
	if( ! defined $gnuplot_file ){
		print STDERR "Warning: no data to plot; maybe device is not connected (to the given port)\n";
		return;
	}

	close FGPLDAT;

	my $gnuplot_cmdfile = 'Ri_DC.gpl';
	open( FGPL, ">$gnuplot_cmdfile");

	my $gnucmd = <<EOF;
plot '$gnuplot_file' using 0:(\$1*0.12329) with lines title 'Ri(10 Ohm)', '$gnuplot_file' using 0:(\$2*4.9/1024/16) with lines title 'DC(V)', '$gnuplot_file' using 0:(\$3/1000) with lines title 'heater_integral(1000)', '$gnuplot_file' using 0:(\$4/2000) with lines title 'pump_integral(2000)', '$gnuplot_file' using 0:(\$5/10) with lines title 'heater_command(10)'

pause -1 "Press Return to exit"

EOF

	print FGPL $gnucmd;
	close FGPL;

	my $syscmd = "gnuplot";
	system( "$syscmd $gnuplot_cmdfile" );
}
# ======
sub datfile_open_smart
{
	if( ! defined $gnuplot_file ){
		$gnuplot_file='Ri_DC.dat';
		open(FGPLDAT, ">$gnuplot_file");
	}
}

#==============
# write to file for gnuplot
sub log_Ri_DC
{
	my @a = @_ ;
	datfile_open_smart();
	# ( $Ri, $DC )
	my $s = join "\t",@a;
	print FGPLDAT $s. "\n";	# "$Ri	$DC\n";
}
######################################################

# beware that argument is different (kind) than processbuffer_short()
sub processbuffer
{
	my $b = shift;

	@array= split(//, $b);
	my $g;
	for($g=3; $g<=45; $g += 3) {   
		SSeSSe($g);
	} # for

	$g += 3; # 51
	processbuffer_short( $g );
}

# prints the last 9 bytes of wbo2log in human readable format
# parameter is index to be used in bufword(): 4 or 51 (apparently)
# GLOBAL @array must be filled by caller
sub processbuffer_short
{
	my ($g) = @_;

	my $s = '';
	my $i;
	for($i=$g; $i<=65; $i++) {   
		my $v = $array[ $i ];

		if( ! defined $v ){
			last;
		}

		$s .= sprintf( "%02x " , ord( $v ));
	} # for
	print "$s\n";

	my $Ri = bufword($g);
    print '# Ri='. $Ri. min_max(\%rimm, $Ri). "\n";

	$g += 2;	
	my $DC = bufword($g);
    print '# DC='. $DC. min_max(\%dcmm, $DC). "\n";

	$g += 2;	
	my $heater_integral = signednum(bufword($g));
    print '# heater_integral='. $heater_integral. min_max(\%himm, $heater_integral). "\n";

	$g += 2;	
	my $pump_integral = signednum(bufword($g));
    print '# pump_integral='. $pump_integral. min_max(\%pimm, $pump_integral). "\n";

	$g += 2;	
	my $heatercommand = ord($array[$g]);
    print '# heatercommand='. $heatercommand. min_max(\%hcmm, $heatercommand). "\n";

	printf("\n\n");

	log_Ri_DC( $Ri, $DC, $heater_integral, $pump_integral, $heatercommand );
}
# =================
sub print_adc
{
	my ($g, $channel_name) = @_;
	my 	$x = bufword_le($g);
	my $voltage = $x * 4.95 / 65536;
	printf("%s:\t%d\t(0x%04x)\t=%02f V\n", $channel_name, $x, $x, $voltage);
}
# GLOBAL @array must be filled by caller
# prints ADC data starting from offset
sub processbuffer_adc
{
	my ($g) = @_;
	
	print_adc( $g, 'VBATT_SENSE');
	print_adc( $g+2*1, 'NERNST_SENSE');
	print_adc( $g+2*2, 'EGT_AMPLIFIED');	
	print_adc( $g+2*3, 'LSU4_PRESENT');
	print_adc( $g+2*4, 'BUTTON_ADC');
	print_adc( $g+2*7, 'NTC_RAW');	
}

###########
# get back word value from 2 bytes of buffer, bigendian assumed
sub bufword
{
	my $g = shift;
	return ord($array[$g])*256 + ord($array[$g+1]);
}
# =====
#little endian
sub bufword_le
{
	my $g = shift;
	return ord($array[$g]) + ord($array[$g+1]) *256;
}
#####
	#the time from OC2 edge (read from TCNT2 in 1/..?.. units) is stored 
	#in edgetime[i]; the result of that sampling will be stored from next 
	#irq invocation to sample[i+1]; 

	#we must send to gnuplot the sample[i+1] - sample[i] 
	#(let's call it jump[i])
	#actually the edgetime[i] jump[i] as x,y pairs
  
	# gnuplot> plot 'file' using $1:abs($2) title '...'

############################
# the buffer we get contains (Sample,Sample,edgetime)...
sub SSeSSe
{
# @array is global (yuck)
	my $g = shift;
	my $sample1 = ord($array[$g])*256 + ord($array[$g+1]);
	my $sample2 = ord($array[$g+3])*256 + ord($array[$g+4]);
	my $jump = $sample2 - $sample1;	# use abs() from gnuplot
	my $edgetime = ord($array[$g-1]);	# from the previous SSe
	print "$edgetime"."\t".$jump."\t".$sample1."\n";
}
###############
sub min_max
{
	my $hashref = shift;
	my $current = shift;
	$hashref->{min} = $current if $current < $hashref->{min};
	$hashref->{max} = $current if $current > $hashref->{max};
	return " min=$hashref->{min} max=$hashref->{max}";
}
################
sub min_max_reset
{
	my $hashref = shift;
	$hashref->{min} = 2000777000;
	$hashref->{max} = -2000777000;
}
#################################
sub process_param
{

#print "number of args:$#ARGV\n";
	$numbuf = 16; # default 
	$mde_exit = "mde00";	# turn off the wbo2
	my $w;
	
	# number of args was checked earlier
   	while($w = shift @ARGV) {
	if ($w =~ /^\d+$/){
		$bootl_speed= $w;
		print "serial speed set to $w\n";
	} elsif ($w =~ /^:(\S+)$/){
		$port = $1;
	} elsif ($w =~ /^\_(\d+)$/){
		$numbuf = $1;
	} elsif ($w =~ /^md[edxP][a-fA-F\d]{2}$/){
		$mde_exit = $w;
	} elsif ($w =~ /f:(.+)/ ){
		$display_file = $1; 
#		   //fileRead($1);			
	}
	} # while

}

#######################
sub comm_send
{
    my $str = shift ;
    $count_out = $ob->write($str);
    warn "write failed\n" unless ($count_out);
    warn "write incomplete\n" if ($count_out != length($str));
    return $count_out;
}

#######################
sub comm_recv
{
    my $b = shift;
    ($count_in, $string_in) = $ob->read($b);
    warn "read unsuccessful\n" unless ($count_in == $b);
    return ($count_in, $string_in);
}

sub flush
{
	my $b = shift;
	$ob->read_const_time($tmout);
	($count_in, $string_in) = $ob->read($b);
	return ($count_in, $string_in);
}


#################################################
# what's the reply???
sub srl_wait
{
	my $realcharcnt = shift;
	if (defined($realcharcnt)) {
	    $charcnt = $realcharcnt;
	} else {
	    $charcnt = 1;
	}
# TODO: if no reply comes within 2 sec, we should start saying 'v' -s or sg.
# finally report the problem (or let the user ctrl-c)
	# $ob->read($in_);
	$ob->read_const_time($tmout);
	my ($cnt, $in_) = comm_recv($charcnt);
	#sysread (SRL, $in_, 1, 0) or die "cannot readback from SRL $!";
	
	if(defined $in_){
		#print "got back $cnt:$in_ :".ord(substr($in_, 0, 1));
		#print ' '.ord(substr($in_, 1, 1)) if(length($in_) > 1);
		#print "\n";
	} else {
		print "nothing\n";
	}
	
	return $in_ if defined($realcharcnt) && ($realcharcnt > 0);
	return undef if $in_ eq "\r";	# success 
	return 1;					# error
}
sub signednum
{
        my $u = shift;
        return $u if $u < 32768;
        return $u - 65535;
}

# Sends char to the display
sub fileRead
{
	my $file = shift;
	my $line;
	
	open F, "< $file";
	while( $line = <F>){
		chomp($line);
		print "Send command: $line \n";
		comm_send($line);		
		sleep(1);		
	}	
	close F;
	exit;
}

#----------------- WBO2 PID TUNING:

# wired in config_t from address 5:
sub config_offset
{
	my $key = shift;
# make_conf.pl ParseConf.pm whatever could be used... 
  if( ! defined $global_config_offsets ){
	$global_config_offsets = {
	wbo2_heater_pid_kp => 5,
	wbo2_heater_pid_ki=>6,
	wbo2_heater_pid_kd=>7,
	wbo2_heater_pid_ilimit=>8,
	wbo2_pump_pid_kp=>9,
	wbo2_pump_pid_ki=>10,
	wbo2_pump_pid_kd=>11,
	wbo2_pump_pid_ilimit =>12
	};
	};

	return $global_config_offsets->{$key};
}

# ============
# uses mttg..mttc.. to adjust a config variable
sub tell_a_var
{
	my ($key, $hexval) = @_;
 
	my $s = sprintf("Manmttg%02xmttc%s", config_offset( $key ), $hexval );
	# print FGPLDAT " # $s\n";
	comm_send( $s );
	return $s;
}

sub tell_conf
{
	my $href = shift;
	my $k;
	foreach $k (sort keys %$href ){
		my $val = $href->{$k} ;
		my $hexval = sprintf("%02x", $val);
		tell_a_var( $k, $hexval );
		print FGPLDAT " \# config.$k=$hexval\n";
	}
}

# cycle through reasonable PID parameters
sub wbo2_PID_cycle
{

	datfile_open_smart();
	
	foreach $wbo2_heater_pid_kp (0x20,0x44,0x60){
	foreach $wbo2_heater_pid_ki (0x06,0x10,0x20){
	foreach $wbo2_heater_pid_kd (0x06, 0x09, 0x10){
	foreach $wbo2_heater_pid_ilimit (0xFF){
	
	foreach $wbo2_pump_pid_kp (0x18, 0x30){
	foreach $wbo2_pump_pid_ki (0x20, 0x28, 0x30){
	foreach $wbo2_pump_pid_kd (0x00){
	foreach $wbo2_pump_pid_ilimit (0xFF){

	tell_conf( {
	wbo2_heater_pid_kp=>$wbo2_heater_pid_kp,
	wbo2_heater_pid_ki=>$wbo2_heater_pid_ki,
	wbo2_heater_pid_kd=>$wbo2_heater_pid_kd,
	wbo2_heater_pid_ilimit=>$wbo2_heater_pid_ilimit,
	wbo2_pump_pid_kp=>$wbo2_pump_pid_kp,
	wbo2_pump_pid_ki=>$wbo2_pump_pid_ki,
	wbo2_pump_pid_kd=>$wbo2_pump_pid_kd,
	wbo2_pump_pid_ilimit =>$wbo2_pump_pid_ilimit
	} );

	# collect_samples:
	wbo2_samples( 'mde03' );
	
	}}}}}}}};	
}
# ========

